
#ifndef __response__
#define __response__

#include "complex.h"
#include "stock.h"

#ifdef	__cplusplus
extern "C" {
#endif

/* the following routines define how to read, write, and print 
 * the filter specific parameters, and how to evaluate the filter
 * at a particular frequency.  Read, write, and print take two
 * arguments: the open stream, and the pointer to the private pvt
 * area.  Read must allocate the memory for the private parameters, 
 * and free can be used to release that memory.  Write must write
 * the parameters in the form such that read understands, 
 * while print may print the filter in a more verbose format.
 * Eval takes two arguments: the (real, positive) frequency, and 
 * a pointer to the private parameters, and returns two values:
 * the amplitude and phase.
 */

typedef struct Response_group {
    int srcid ; 	
    int sequence ; 	/* sequence number ( > 1 ) */
    char description[13] ; /* ascii description */
    int id ;		
    char author[45] ;   /* source of response information */
    int  apply ; 	/* if non-zero, include this filter in the response */
    Tbl *comments ; 
    Arr *parameters ;
    void *pvt ; 	/* internal parameters for this particular filter */
    } Response_group ; 

typedef struct Response {
    int ngroups ; 
    int nmax ; 
    Response_group *groups ; 
    char *dfile ;	/* associated file name */
    char *insname ;	/* associated instrument name */
    } Response ;

typedef struct Fir {
    int nnum, nden ;
    double srate ; 
    int dec_factor ;
    int seed_dec_offset ;
    double midpoint ; 
    double *num_coefs, *num_coef_errors ;
    double *den_coefs, *den_coef_errors ;
    } Fir ;

typedef struct Fap {
    int ntriplets;
    double *freqs;
    double *amps, *amp_errors ;
    double *phases, *phase_errors ;
    } Fap ; 

typedef struct Fap2 {
    int ntriplets;
    double *freqs;
    double *amps, *amp_errors_high, *amp_errors_low ;
    double *phases, *phase_errors_high, *phase_errors_low ;
    } Fap2 ; 

typedef struct Paz {
    int npoles, nzeros ; 
    double normalization, frequency ;
    Complex_t *poles, *pole_errors ; 
    Complex_t *zeros, *zero_errors ; 
    } Paz ; 

typedef struct Iir {
    int npoles, nzeros ; 
    double normalization, frequency ;
    Complex_t *poles, *pole_errors ; 
    Complex_t *zeros, *zero_errors ; 
    } Iir ; 
 
#define TWO_FLOATS "[ \t]*(-?([0-9]*)?\\.([0-9]*)?([eE][+-]?[0-9]+)?|-?[0-9]+([eE][+-]?[0-9]+)?)[ \t]+(-?([0-9]*)?\\.([0-9]*)?([eE][+-]?[0-9]+)?|-?[0-9]+([eE][+-]?[0-9]+)?)[ \t]*"
#define ONE_FLOAT "[ \t]*(-?([0-9]*)?\\.([0-9]*)?([eE][+-]?[0-9]+)?|-?[0-9]+([eE][+-]?[0-9]+)?)[ \t]*"
#define ONE_INT    "[ \t]*[0-9]+[ \t]*(Poles|Zeros)?[ \t]*"
#define POLES    "[ \t]*[0-9]+[ \t]*(Poles)?[ \t]*"
#define ZEROS    "[ \t]*[0-9]+[ \t]*(Zeros)?[ \t]*"
#define TYPE_LINE  "(theoretical|measured|unknown)[ \t]+[0-9]+[ \t]+[-a-zA-Z_0-9]+[ \t]+(paz|fir|iir|fap|fap2)[ \t]*.*"
#define FIR_SAMPRATE "[ \t]*(-?([0-9]*)?\\.([0-9]*)?([eE][+-]?[0-9]+)?|-?[0-9]+([eE][+-]?[0-9]+)?)[ \t]*[0-9]+[ \t]*((-?([0-9]*)?\\.([0-9]*)?([eE][+-]?[0-9]+)?|-?[0-9]+([eE][+-]?[0-9]+)?)[ \t]*)?"
 
/* These routines form an interface to the instrument response files */
 
extern void init_response_group ( Response_group *gp );
extern Response_group *new_response_group ( void );
extern Response *new_response ( void );
extern int add_response_group ( Response *rp, Response_group *gp );
extern int copy_response_group ( Response_group *gp, Response *rp, int indx );
extern int read_response ( FILE *file, Response **response );
extern int write_response ( FILE *file, Response *response );
extern int print_response ( FILE *file, Response *response );
extern int eval_response ( double omega, Response *response, double *real, double *imag );
extern int normalize_response ( Response *response, double omega );
extern void free_response ( Response *response );
extern int compare_response ( Response *response1, Response *response2 );
extern int get_response_nstages ( Response *response, int *nstages );
extern int get_response_stage_type ( Response *response, int stage, char *type );
extern int get_response_stage_fir_ncoefs ( Response *response, int stage, double *srate, int *dec_factor, int *nnum, int *nden );
extern int get_response_stage_fir_coefs ( Response *response, int stage, int *nnum, double **coefn, double **coefn_err, int *nden, double **coefd, double **coefd_err );
extern void readresp_ ( char *fname, Response **response, int lenfname );
extern void evlresp_ ( float *omega, Response **response, float *real, float *imag );
extern void freeresp_ ( Response **response );
extern void getrespnstages_ ( Response **response, int *nstages );
extern void getresptype_ ( Response **response, int *stage, char *type, int lentype );
extern void getresppaz_ ( Response **response, int *stage, float *norm, int *npoles, float *poles, float *polerr, int *nzeroes, float *zeroes, float *zererr );
extern void getrespfir_ ( Response **response, int *stage, float *srate, int *nnum, float *coefsnum, float *coefsnumerr, int *nden, float *coefsden, float *coefsdenerr );
extern void getrespfap_ ( Response **response, int *stage, int *ntriplets, float *freqs, float *amps, float *amperr, float *phases, float *phaseerr );
extern void fixresp ( Response *response, double output_sample_rate );
extern void fixsrate ( Response *response, double output_sample_rate );
extern void getrespiir_ ( Response **response, int *stage, float *norm, int *npoles, float *poles, float *polerr, int *nzeroes, float *zeroes, float *zererr );

extern void to_polar ( double real, double imag, double *amp, double *phase );
extern void to_complex ( double amp, double phase, double *real, double *imag );
extern Complex_t cmult ( Complex_t a, Complex_t b );
extern Complex_t cdiv ( Complex_t a, Complex_t b );

extern int brack_ ( int *n, float *x, float *x0, int *ileft );
extern void spline ( float x[], float y[], int n, double yp1, double ypn, float y2[] );
extern void splint ( float xa[], float ya[], float y2a[], int n, double x, float *y );

extern int read_paz ( FILE *file, Response_group *gp );
extern int write_paz ( FILE *file, Paz *pvt );
extern int print_paz ( FILE *file, Paz *pvt );
extern void free_paz ( Paz *pvt );
extern int eval_paz ( double omega, Paz *pvt, double *amplitude, double *phase );
extern int norm_paz ( double omega, Paz *pvt );
extern int compare_paz ( Paz *paz1, Paz *paz2 );

#ifdef	__cplusplus
}
#endif

#define PAZ 0 
#define FAP 1
#define FIR 2
#define IIR 3
#define FAP2 4

#define THEORETICAL 0 
#define MEASURED 1

#define E_BADFILE 501

#endif


/* $Id: response.h,v 1.12 2003/10/01 23:42:04 danq Exp $ */
